@@ -27,8 +27,7 @@
 
 #define MAXBACKTRACE 20
 
-#if defined(CONFIG_STACKTRACE) && LINUX_VERSION_CODE > KERNEL_VERSION(2,6,26)
-// XXX: PR9866: hacky temporarily restriction to recent kernels
+#if defined(STAPCONF_KERNEL_STACKTRACE)
 #include <linux/stacktrace.h>
 #include <asm/stacktrace.h>
 #endif
@@ -51,7 +50,7 @@ static void _stp_stack_print_fallback(unsigned long, int, int);
 #error "Unsupported architecture"
 #endif
 
-#if defined(CONFIG_STACKTRACE) && LINUX_VERSION_CODE > KERNEL_VERSION(2,6,26)
+#if defined(STAPCONF_KERNEL_STACKTRACE)
 
 struct print_stack_data
 {
@@ -161,7 +160,7 @@ static void _stp_ustack_print(char *str)
 
 void _stp_stack_print_tsk(struct task_struct *tsk, int verbose, int levels)
 {
-#if defined(CONFIG_STACKTRACE) && LINUX_VERSION_CODE > KERNEL_VERSION(2,6,26)
+#if defined(STAPCONF_KERNEL_STACKTRACE)
         int i;
         unsigned long backtrace[MAXBACKTRACE];
         struct stack_trace trace;
