@@ -16,18 +16,6 @@
 #include <grp.h>
 #include <pwd.h>
 
-void setup_staprun_signals(void)
-{
-	struct sigaction a;
-	memset(&a, 0, sizeof(a));
-	sigfillset(&a.sa_mask);
-	a.sa_handler = SIG_IGN;
-	sigaction(SIGINT, &a, NULL);
-	sigaction(SIGTERM, &a, NULL);
-	sigaction(SIGHUP, &a, NULL);
-	sigaction(SIGQUIT, &a, NULL);
-}
-
 extern long init_module(void *, unsigned long, const char *);
 
 /* Module errors get translated. */
@@ -55,7 +43,7 @@ int insert_module(const char *path, const char *special_options, char **options)
 	char *opts;
 	int fd, saved_errno;
 	struct stat sbuf;
-		
+
 	dbug(2, "inserting module\n");
 
 	if (special_options)
@@ -83,7 +71,7 @@ int insert_module(const char *path, const char *special_options, char **options)
 		perr("Error opening '%s'", path);
 		return -1;
 	}
-	
+
 	/* Now that the file is open, figure out how big it is. */
 	if (fstat(fd, &sbuf) < 0) {
 		_perr("Error stat'ing '%s'", path);
@@ -99,9 +87,9 @@ int insert_module(const char *path, const char *special_options, char **options)
 		free(opts);
 		return -1;
 	}
-	    
+
 	/* Actually insert the module */
-	ret = do_cap(CAP_SYS_MODULE, init_module, file, sbuf.st_size, opts);
+	ret = init_module(file, sbuf.st_size, opts);
 	saved_errno = errno;
 
 	/* Cleanup. */
@@ -111,7 +99,7 @@ int insert_module(const char *path, const char *special_options, char **options)
 
 	if (ret != 0) {
 		err("Error inserting module '%s': %s\n", path, moderror(saved_errno));
-		return -1; 
+		return -1;
 	}
 	return 0;
 }
@@ -132,8 +120,7 @@ int mountfs(void)
 	rc = stat(DEBUGFSDIR, &sb);
 	if (rc == 0 && S_ISDIR(sb.st_mode)) {
 		/* If we can mount the debugfs dir correctly, we're done. */
-		rc = do_cap(CAP_SYS_ADMIN, mount, "debugfs", DEBUGFSDIR,
-			    "debugfs", 0, NULL); 
+          	rc = mount ("debugfs", DEBUGFSDIR, "debugfs", 0, NULL);
 		if (rc == 0)
 			return 0;
 		/* If we got ENODEV, that means that debugfs isn't
@@ -144,7 +131,7 @@ int mountfs(void)
 			return -1;
 		}
 	}
-	
+
 	/* DEBUGFSDIR couldn't be mounted.  So, try RELAYFSDIR. */
 
 	/* If the relayfs dir is already mounted correctly, we're done. */
@@ -171,11 +158,12 @@ int mountfs(void)
 		/* To ensure the directory gets created with the
 		 * proper group, we'll have to temporarily switch to
 		 * root. */
-		if (do_cap(CAP_SETUID, setuid, 0) < 0) {
+                /* XXX: Why not just chown() the thing? */
+		if (setuid (0) < 0) {
 			_perr("Couldn't change user while creating %s", RELAYFSDIR);
 			return -1;
 		}
-		if (do_cap(CAP_SETGID, setgid, 0) < 0) {
+		if (setgid (0) < 0) {
 			_perr("Couldn't change group while creating %s", RELAYFSDIR);
 			return -1;
 		}
@@ -186,11 +174,11 @@ int mountfs(void)
 		saved_errno = errno;
 
 		/* Restore everything we changed. */
-		if (do_cap(CAP_SETGID, setgid, gid) < 0) {
+		if (setgid (gid) < 0) {
 			_perr("Couldn't restore group while creating %s", RELAYFSDIR);
 			return -1;
 		}
-		if (do_cap(CAP_SETUID, setuid, uid) < 0) {
+		if (setuid (uid) < 0) {
 			_perr("Couldn't restore user while creating %s", RELAYFSDIR);
 			return -1;
 		}
@@ -204,7 +192,7 @@ int mountfs(void)
 	}
 
 	/* Now that we're sure the directory exists, try mounting RELAYFSDIR. */
-	if (do_cap(CAP_SYS_ADMIN, mount, "relayfs", RELAYFSDIR, "relayfs", 0, NULL) < 0) {
+	if (mount ("relayfs", RELAYFSDIR, "relayfs", 0, NULL) < 0) {
 		perr("Couldn't mount %s", RELAYFSDIR);
 		return -1;
 	}
@@ -274,13 +262,22 @@ check_path(void)
 		     "  Unable to canonicalize that directory",	staplib_dir_path);
 		return -1;
 	}
-	
+
 	/* Use realpath() to canonicalize the module path. */
 	if (realpath(modpath, module_realpath) == NULL) {
 		perr("Unable to canonicalize path \"%s\"", modpath);
 		return -1;
 	}
-	
+
+        /* Overwrite the modpath with the canonicalized one, to defeat
+           a possible race between path checking below and somewhat later
+           module loading. */
+        modpath = strdup (module_realpath);
+        if (modpath == NULL) {
+		_perr("allocating memory failed");
+                exit (1);
+        }
+
 	/* To make sure the user can't specify something like
 	 * /lib/modules/`uname -r`/systemtapmod.ko, put a '/' on the
 	 * end of staplib_dir_realpath. */
@@ -319,7 +316,7 @@ int check_permissions(void)
 	int path_check = 0;
 
 	/* If we're root, we can do anything. */
-	if (geteuid() == 0)
+	if (getuid() == 0)
 		return 1;
 
 	/* Lookup the gid for group "stapdev" */
@@ -401,95 +398,3 @@ int check_permissions(void)
 	 * is in that directory. */
 	return check_path();
 }
-
-pthread_t symbol_thread_id = (pthread_t)0;
-int kernel_ptr_size = 0;
-
-/* Symbol handling thread */
-void *handle_symbols(void __attribute__((unused)) *arg)
-{
-	ssize_t nb;
-	void *data;
-	int32_t type;
-	char recvbuf[8192];
-
-	dbug(2, "waiting for symbol requests\n");
-
-	/* handle messages from control channel */
-	while (1) {
-		nb = read(control_channel, recvbuf, sizeof(recvbuf));
-		if (nb <= 0) {
-			if (errno != EINTR)
-				_perr("Unexpected EOF in read (nb=%ld)", (long)nb);
-			continue;
-		}
-		
-		type = *(int32_t *)recvbuf;
-		data = (void *)(recvbuf + sizeof(int32_t));
-		
-		switch (type) { 
-		case STP_MODULE:
-		{
-			dbug(2, "STP_MODULES request received\n");
-			if (do_module(data) < 0)
-				goto done;
-			break;
-		}		
-		case STP_SYMBOLS:
-		{
-			struct _stp_msg_symbol *req = (struct _stp_msg_symbol *)data;
-			dbug(2, "STP_SYMBOLS request received\n");
-			if (req->endian != 0x1234) {
-				err("ERROR: staprun is compiled with different endianess than the kernel!\n");
-				goto done;
-			}
-			kernel_ptr_size = req->ptr_size;
-			if (kernel_ptr_size != 4 && kernel_ptr_size != 8) {
-				err("ERROR: invalid kernel pointer size %d\n", kernel_ptr_size);
-				goto done;
-			}
-			if (do_kernel_symbols() < 0)
-				goto done;
-			break;
-		}
-		default:
-			err("WARNING: ignored message of type %d\n", (type));
-		}
-	}
-
-done:
-	/* signal stapio we're done */
-	kill(0, SIGINT);
-
-	return NULL;
-}
-
-void start_symbol_thread(void)
-{
-	int status;
-
-	/* create symbol control channel */
-	status = do_cap(CAP_DAC_OVERRIDE, init_ctl_channel, 1);
-	drop_cap(CAP_DAC_OVERRIDE);
-	if (status < 0) {
-		err("Failed to initialize control channel.\n");
-		exit(1);
-	}
-	status = pthread_create(&symbol_thread_id, NULL, handle_symbols, NULL);
-	if (status) {
-		perr("Failed to create symbol thread.\n");
-		exit(1);
-	}
-}
-
-void stop_symbol_thread(void)
-{
-
-	if (symbol_thread_id) {
-		dbug(2, "Stopping symbol thread.\n");
-		pthread_cancel(symbol_thread_id);
-		pthread_join(symbol_thread_id, NULL);
-	}
-	close_ctl_channel();
-}
-
