@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * String Functions
- * Copyright (C) 2005, 2006, 2007 Red Hat Inc.
+ * Copyright (C) 2005, 2006, 2007, 2009 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -28,7 +28,7 @@
  * variable number of args.
  */
 
-int _stp_snprintf(char *buf, size_t size, const char *fmt, ...)
+static int _stp_snprintf(char *buf, size_t size, const char *fmt, ...)
 {
         va_list args;
         int i;
@@ -39,7 +39,7 @@ int _stp_snprintf(char *buf, size_t size, const char *fmt, ...)
         return i;
 }
 
-int _stp_vscnprintf(char *buf, size_t size, const char *fmt, va_list args)
+static int _stp_vscnprintf(char *buf, size_t size, const char *fmt, va_list args)
 {
 	unsigned i = _stp_vsnprintf(buf,size,fmt,args);
 	return (i >= size) ? (size - 1) : i;
@@ -60,30 +60,30 @@ int _stp_vscnprintf(char *buf, size_t size, const char *fmt, va_list args)
  * in will have "..." after the second quote.
  * @param user Set this to indicate the input string pointer is a userspace pointer.
  */
-void _stp_text_str(char *outstr, char *in, int len, int quoted, int user)
+static void _stp_text_str(char *outstr, char *in, int len, int quoted, int user)
 {
-	const int length = len;
 	char c, *out = outstr;
 
 	if (len == 0 || len > MAXSTRINGLEN-1)
 		len = MAXSTRINGLEN-1;
 	if (quoted) {
 		len -= 2;
-		*out++ = '\"';
+		*out++ = '"';
 	}
 
 	if (user) {
 		if (!access_ok(VERIFY_READ, (char __user *)in, 1))
 			goto bad;
-		if (__get_user(c, in))
+		if (__stp_get_user(c, in))
 			goto bad;
 	} else
 		c = *in;
 
 	while (c && len > 0) {
 		int num = 1;
-		if (isprint(c) && isascii(c))
-			*out++ = c;
+		if (isprint(c) && isascii(c)
+                    && c != '"' && c != '\\') /* quoteworthy characters */
+                  *out++ = c;
 		else {
 			switch (c) {
 			case '\a':
@@ -93,6 +93,8 @@ void _stp_text_str(char *outstr, char *in, int len, int quoted, int user)
 			case '\r':
 			case '\t':
 			case '\v':
+			case '"':
+			case '\\':
 				num = 2;
 				break;
 			default:
@@ -131,6 +133,12 @@ void _stp_text_str(char *outstr, char *in, int len, int quoted, int user)
 			case '\v':
 				*out++ = 'v';
 				break;
+			case '"':
+				*out++ = '"';
+				break;
+			case '\\':
+				*out++ = '\\';
+				break;
 			default:                  /* output octal representation */
 				if (c > 077)
 					*out++ = to_oct_digit(c >> 6);
@@ -143,7 +151,7 @@ void _stp_text_str(char *outstr, char *in, int len, int quoted, int user)
 		len -= num;
 		in++;
 		if (user) {
-			if (__get_user(c, in))
+			if (__stp_get_user(c, in))
 				goto bad;
 		} else
 			c = *in;
@@ -152,12 +160,12 @@ void _stp_text_str(char *outstr, char *in, int len, int quoted, int user)
 	if (quoted) {
 		if (c) {
 			out = out - 3 + len;
-			*out++ = '\"';
+			*out++ = '"';
 			*out++ = '.';
 			*out++ = '.';
 			*out++ = '.';
 		} else
-			*out++ = '\"';
+			*out++ = '"';
 	}
 	*out = '\0';
 	return;
